@@ -14,6 +14,8 @@ import sys, csv, re, argparse, os, zipfile, io
 import logging
 from collections import Counter, OrderedDict
 
+import copy
+
 
 # Python 2 compatibility
 if sys.version_info.major == 2:
@@ -57,9 +59,10 @@ class WesException(Exception):
 
 
 # Applictation details
-VERSION = 0.95
+VERSION = 0.98
+RELEASE = ''
 WEB_URL = 'https://github.com/bitsadmin/wesng/'
-BANNER = 'Windows Exploit Suggester %.2f ( %s )' % (VERSION, WEB_URL)
+BANNER = 'Windows Exploit Suggester %.2f%s ( %s )' % (VERSION, RELEASE, WEB_URL)
 FILENAME = 'wes.py'
 
 # Mapping table between build numbers and versions to correctly identify
@@ -71,7 +74,8 @@ buildnumbers = OrderedDict([
     (15063, 1703),
     (16299, 1709),
     (17134, 1803),
-    (17763, 1809)
+    (17763, 1809),
+    (18362, 1903)
 ])
 
 
@@ -160,6 +164,7 @@ def main():
 
     # Append manually specified KBs to list of hotfixes
     hotfixes = list(set(hotfixes + manual_hotfixes))
+    hotfixes_orig = copy.deepcopy(hotfixes)
 
     # Load definitions from definitions.zip (default) or user-provided location
     print('[+] Loading definitions')
@@ -190,12 +195,20 @@ def main():
 
     # If specified, hide results containing the user-specified string
     # in the AffectedComponent and AffectedProduct attributes
-    if args.hiddenvuln or args.only_exploits or args.impacts:
+    if args.hiddenvuln or args.only_exploits or args.impacts or args.severities:
         print('[+] Applying display filters')
-        filtered = apply_display_filters(found, args.hiddenvuln, args.only_exploits, args.impacts)
+        filtered = apply_display_filters(found, args.hiddenvuln, args.only_exploits, args.impacts, args.severities)
     else:
         filtered = found
 
+    # If specified, lookup superseeding KBs in the Microsoft Update Catalog
+    # and remove CVEs if a superseeding KB is installed.
+    if args.muc_lookup:
+        from muc_lookup import apply_muc_filter # ony import if necessary since it needs MechanicalSoup
+
+        print("[+] Looking up superseeding hotfixes in the Microsoft Update Catalog")
+        filtered = apply_muc_filter(filtered, hotfixes_orig)
+
     # Split up list of KBs and the potential Service Packs/Cumulative updates available
     kbs, sp = get_patches_servicepacks(filtered, cves, productfilter)
 
@@ -256,10 +269,11 @@ def load_definitions(definitions):
 # Hide results based on filter(s) specified by the user. This can either be to only display results with
 # public exploits, results with a given impact or results containing the user specified string(s) in
 # the AffectedComponent or AffectedProduct attributes.
-def apply_display_filters(found, hiddenvulns, only_exploits, impacts):
+def apply_display_filters(found, hiddenvulns, only_exploits, impacts, severities):
     # --hide 'Product 1' 'Product 2'
     hiddenvulns = list(map(lambda s: s.lower(), hiddenvulns))
     impacts = list(map(lambda s: s.lower(), impacts))
+    severities = list(map(lambda s: s.lower(), severities))
     filtered = []
     for cve in found:
         add = True
@@ -271,6 +285,15 @@ def apply_display_filters(found, hiddenvulns, only_exploits, impacts):
         for impact in impacts:
             if not impact in cve['Impact'].lower():
                 add = False
+            else:
+                add = True
+                break
+
+        for severity in severities:
+            if not severity in cve['Severity'].lower():
+                add = False
+            else:
+                add = True
                 break
 
         if add:
@@ -394,8 +417,11 @@ def mark_superseeded_hotfix(filtered, superseeded, marked):
 def determine_product(systeminfo):
     systeminfo = charset_convert(systeminfo)
 
+    # Fixup for 7_sp1_x64_enterprise_fr_systeminfo_powershell.txt
+    systeminfo = systeminfo.replace('\xA0', '\x20')
+
     # OS Version
-    regex_version = re.compile(r'.*?((\d+\.?)+) ((Service Pack (\d)|N\/\w|.+) )?[ -\xa5]+ (\d+).*', re.MULTILINE | re.IGNORECASE)
+    regex_version = re.compile(r'.*?((\d+\.?){3}) ((Service Pack (\d)|N\/\w|.+) )?[ -\xa5]+ (\d+).*', re.MULTILINE | re.IGNORECASE)
     systeminfo_matches = regex_version.findall(systeminfo)
     if len(systeminfo_matches) == 0:
         raise WesException('Not able to detect OS version based on provided input file')
@@ -405,13 +431,18 @@ def determine_product(systeminfo):
     servicepack = systeminfo_matches[4]
 
     # OS Name
-    win_matches = re.findall('.*?Microsoft[\(R\)]{0,3} Windows[\(R\)]{0,3} (Serverr? )?(\d+\.?\d?( R2)?|XP|VistaT).*', systeminfo, re.MULTILINE | re.IGNORECASE)
+    win_matches = re.findall('.*?Microsoft[\(R\)]{0,3} Windows[\(R\)?]{0,3} ?(Serverr? )?(\d+\.?\d?( R2)?|XP|VistaT).*', systeminfo, re.MULTILINE | re.IGNORECASE)
     if len(win_matches) == 0:
         raise WesException('Not able to detect OS name based on provided input file')
     win = win_matches[0][1]
 
     # System Type
-    arch = re.findall('.*?([\w\d]+?)-based PC.*', systeminfo, re.MULTILINE | re.IGNORECASE)[0]
+    archs = re.findall('.*?([\w\d]+?)-based PC.*', systeminfo, re.MULTILINE | re.IGNORECASE)
+    if len(archs) > 0:
+        arch = archs[0]
+    else:
+        logging.warning('Cannot determine system\'s architecture. Assuming x64')
+        arch = 'x64'
 
     # Hotfix(s)
     hotfixes = get_hotfixes(systeminfo)
@@ -626,15 +657,21 @@ Impact: %s
 # Output results of wes.py to a .csv file
 def store_results(outputfile, results):
     print('[+] Writing %d results to %s' % (len(results), outputfile))
-    with open(outputfile, 'w', newline='') as f:
-        header = list(results[0].keys())
-        header.remove('Supersedes')
-        writer = csv.DictWriter(f, fieldnames=header, quoting=csv.QUOTE_ALL)
-        writer.writeheader()
-        for r in results:
-            if 'Supersedes' in r:
-                del r['Supersedes']
-            writer.writerow(r)
+
+    # Python 2 compatibility
+    if sys.version_info.major == 2:
+        f = open(outputfile, 'wb')
+    else:
+        f = open(outputfile, 'w', newline='')
+
+    header = list(results[0].keys())
+    header.remove('Supersedes')
+    writer = csv.DictWriter(f, fieldnames=header, quoting=csv.QUOTE_ALL)
+    writer.writeheader()
+    for r in results:
+        if 'Supersedes' in r:
+            del r['Supersedes']
+        writer.writerow(r)
 
 
 # Validate file existence for user-provided arguments
@@ -681,13 +718,20 @@ def parse_arguments():
   Determine vulnerabilities explicitly specifying definitions file
   {0} systeminfo.txt --definitions C:\tmp\mydefs.zip
 
-  List only vulnerabilities with exploits, excluding Edge and Flash
+  List only vulnerabilities with exploits, excluding IE, Edge and Flash
   {0} systeminfo.txt --exploits-only --hide "Internet Explorer" Edge Flash
   {0} systeminfo.txt -e --hide "Internet Explorer" Edge Flash
 
-  Only show vulnerabilities of a certain impact (case insensitive match)
-  {0} systeminfo.txt --impact elevation
-  {0} systeminfo.txt -i elevation
+  Only show vulnerabilities of a certain impact
+  {0} systeminfo.txt --impact "Remote Code Execution"
+  {0} systeminfo.txt -i "Remote Code Execution"
+  
+  Only show vulnerabilities of a certain severity
+  {0} systeminfo.txt --severity critical
+  {0} systeminfo.txt -s critical
+  
+  Validate supersedence against Microsoft's online Update Catalog
+  {0} systeminfo.txt --muc-lookup
 
   Download latest version of WES-NG
   {0} --update-wes
@@ -737,7 +781,10 @@ def parse_arguments():
     parser.add_argument('-e', '--exploits-only', dest='only_exploits', action='store_true', help='Show only vulnerabilities with known exploits')
     parser.add_argument('--hide', dest='hiddenvuln', nargs='+', default='', help='Hide vulnerabilities of for example Adobe Flash Player and Microsoft Edge')
     parser.add_argument('-i', '--impact', dest='impacts', nargs='+', default='', help='Only display vulnerabilities with a given impact')
+    parser.add_argument('-s', '--severity', dest='severities', nargs='+', default='', help='Only display vulnerabilities with a given severity')
     parser.add_argument('-o', '--output', action='store', dest='outputfile', nargs='?', help='Store results in a file')
+    parser.add_argument("--muc-lookup", dest="muc_lookup", action="store_true", help="Hide vulnerabilities if installed hotfixes are listed in the Microsoft Update Catalog as superseding hotfixes for the original BulletinKB",
+    )
     parser.add_argument('-h', '--help', action='help', help='Show this help message and exit')
 
     # Always show full help when no arguments are provided
